@@ -20,6 +20,10 @@
 
 /* This is where functions related to the standalone message window are kept */
 
+// from MailNewsTypes.h
+const nsMsgKey_None = 0xFFFFFFFF;
+const nsMsgViewIndex_None = 0xFFFFFFFF;
+
 /* globals for a particular window */
 
 var compositeDataSourceContractID        = datasourceContractIDPrefix + "composite-datasource";
@@ -30,6 +34,9 @@ var gCurrentFolderUri;
 var gThreadPaneCommandUpdater = null;
 var gCurrentMessageIsDeleted = false;
 var gNextMessageViewIndexAfterDelete = -2;
+var gCurrentFolderToRerootForStandAlone;
+var gRerootOnFolderLoadForStandAlone = false;
+var gNextMessageAfterLoad = null;
 
 // the folderListener object
 var folderListener = {
@@ -37,11 +44,11 @@ var folderListener = {
 
   OnItemRemoved: function(parentItem, item, view) {
     var parentFolderResource = parentItem.QueryInterface(Components.interfaces.nsIRDFResource);
-    if(!parentFolderResource)
+    if (!parentFolderResource)
       return;
 
     var parentURI = parentFolderResource.Value;
-    if(parentURI != gCurrentFolderUri)
+    if (parentURI != gCurrentFolderUri)
       return;
 
     var deletedMessageHdr = item.QueryInterface(Components.interfaces.nsIMsgDBHdr);
@@ -50,24 +57,110 @@ var folderListener = {
   },
 
   OnItemPropertyChanged: function(item, property, oldValue, newValue) {},
-  OnItemIntPropertyChanged: function(item, property, oldValue, newValue) { },
+  OnItemIntPropertyChanged: function(item, property, oldValue, newValue) { 
+    var currentLoadedFolder = GetLoadedMsgFolder();
+    if (!currentLoadedFolder) return;
+    var currentURI = currentLoadedFolder.URI;
+
+    //if we don't have a folder loaded, don't bother.
+    if (currentURI) {
+      if (property.toString() == "TotalMessages" || property.toString() == "TotalUnreadMessages") {
+        var folder = item.QueryInterface(Components.interfaces.nsIMsgFolder);
+        if (folder) {
+          var folderResource = folder.QueryInterface(Components.interfaces.nsIRDFResource); 
+          if (folderResource) {
+            var folderURI = folderResource.Value;
+            if (currentURI == folderURI) {
+              UpdateStandAloneMessageCounts();
+            }
+          }
+        }
+      }      
+    }
+  },
   OnItemBoolPropertyChanged: function(item, property, oldValue, newValue) {},
   OnItemUnicharPropertyChanged: function(item, property, oldValue, newValue){},
   OnItemPropertyFlagChanged: function(item, property, oldFlag, newFlag) {},
 
   OnItemEvent: function(folder, event) {
-    if (event.GetUnicode() == "DeleteOrMoveMsgCompleted") {
+    var eventType = event.toString();
+
+    if (eventType == "DeleteOrMoveMsgCompleted")
       HandleDeleteOrMoveMsgCompleted(folder);
-    }     
-    else if (event.GetUnicode() == "DeleteOrMoveMsgFailed") {
+    else if (eventType == "DeleteOrMoveMsgFailed")
       HandleDeleteOrMoveMsgFailed(folder);
+    else if (eventType == "FolderLoaded") {
+      if (folder) {
+        var uri = folder.URI;
+        if (uri == gCurrentFolderToRerootForStandAlone) {
+          gCurrentFolderToRerootForStandAlone = null;
+          var msgFolder = folder.QueryInterface(Components.interfaces.nsIMsgFolder);
+          if (msgFolder) {
+            msgFolder.endFolderLoading();
+            if (gRerootOnFolderLoadForStandAlone) {
+              RerootFolderForStandAlone(uri);
+            }
+          }   
+        }
+      }
     }
-  }
+    else if (eventType == "JunkStatusChanged") {
+      HandleJunkStatusChanged(folder);
+    }
+  }   
 }
 
+var messagepaneObserver = {
+
+  canHandleMultipleItems: false,
+
+  onDrop: function (aEvent, aData, aDragSession)
+  {
+    var sourceUri = aData.data; 
+    if (sourceUri != gCurrentMessageUri)
+    {
+      SelectFolder(GetMsgHdrFromUri(sourceUri).folder.URI);
+      SelectMessage(sourceUri);
+    }
+  },
+ 
+  onDragOver: function (aEvent, aFlavour, aDragSession)
+  {
+    var messagepanebox = document.getElementById("messagepanebox");
+    messagepanebox.setAttribute("dragover", "true");
+  },
+ 
+  onDragExit: function (aEvent, aDragSession)
+  {
+    var messagepanebox = document.getElementById("messagepanebox");
+    messagepanebox.removeAttribute("dragover");
+  },
+
+  canDrop: function(aEvent, aDragSession)  //allow drop from mail:3pane window only - 4xp
+  {
+    var doc = aDragSession.sourceNode.ownerDocument;
+    var elem = doc.getElementById("messengerWindow");
+    return (elem && (elem.getAttribute("windowtype") == "mail:3pane"));
+  },
+  
+  getSupportedFlavours: function ()
+  {
+    var flavourSet = new FlavourSet();
+    flavourSet.appendFlavour("text/x-moz-message");
+    return flavourSet;
+  }
+};
+
 function nsMsgDBViewCommandUpdater()
 {}
 
+function UpdateStandAloneMessageCounts()
+{
+  // hook for extra toolbar items
+  var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
+  observerService.notifyObservers(window, "mail:updateStandAloneMessageCounts", "");
+}
+
 nsMsgDBViewCommandUpdater.prototype = 
 {
   updateCommandStatus : function()
@@ -77,41 +170,45 @@ nsMsgDBViewCommandUpdater.prototype =
       UpdateMailToolbar("dbview, std alone window");
     },
 
-  displayMessageChanged : function(aFolder, aSubject)
+  displayMessageChanged : function(aFolder, aSubject, aKeywords)
   {
     setTitleFromFolder(aFolder, aSubject);
     gCurrentMessageUri = gDBView.URIForFirstSelectedMessage;
+    UpdateStandAloneMessageCounts();
+    SetKeywords(aKeywords);
+    goUpdateCommand("button_junk");
+  },
+
+  updateNextMessageAfterDelete : function()
+  {
+    SetNextMessageAfterDelete();
   },
 
   QueryInterface : function(iid)
   {
-    if(iid.equals(Components.interfaces.nsIMsgDBViewCommandUpdater))
-	    return this;
+    if (iid.equals(Components.interfaces.nsIMsgDBViewCommandUpdater) ||
+        iid.equals(Components.interfaces.nsISupports))
+      return this;
 	  
     throw Components.results.NS_NOINTERFACE;
-    return null;
   }
 }
 
-// from MailNewsTypes.h
-const nsMsgKey_None = 0xFFFFFFFF;
-
 function HandleDeleteOrMoveMsgCompleted(folder)
 {
-	dump("In HandleDeleteOrMoveMsgCompleted\n");
 	var folderResource = folder.QueryInterface(Components.interfaces.nsIRDFResource);
-	if(!folderResource)
+	if (!folderResource)
 		return;
 
-	var folderUri = folderResource.Value;
-	if((folderUri == gCurrentFolderUri) && gCurrentMessageIsDeleted)
+	if ((folderResource.Value == gCurrentFolderUri) && gCurrentMessageIsDeleted)
 	{
+    gDBView.onDeleteCompleted(true);
     gCurrentMessageIsDeleted = false;
-    if (gNextMessageViewIndexAfterDelete != nsMsgKey_None) 
+    if (gNextMessageViewIndexAfterDelete != nsMsgViewIndex_None) 
     {
       var nextMstKey = gDBView.getKeyAt(gNextMessageViewIndexAfterDelete);
       if (nextMstKey != nsMsgKey_None) {
-        gDBView.loadMessageByMsgKey(nextMstKey);
+        LoadMessageByMsgKey(nextMstKey);
       }
       else {
         window.close();
@@ -128,22 +225,31 @@ function HandleDeleteOrMoveMsgCompleted(folder)
 function HandleDeleteOrMoveMsgFailed(folder)
 {
   var folderResource = folder.QueryInterface(Components.interfaces.nsIRDFResource);
-  if(!folderResource)
+  if (!folderResource)
      return;
 
-  var folderUri = folderResource.Value;
-  if((folderUri == gCurrentFolderUri) && gCurrentMessageIsDeleted)
-  {
+  gDBView.onDeleteCompleted(false);
+  if ((folderResource.Value == gCurrentFolderUri) && gCurrentMessageIsDeleted)
     gCurrentMessageIsDeleted = false;
-  }	
+}
+
+function IsCurrentLoadedFolder(folder)
+{
+  var folderResource = folder.QueryInterface(Components.interfaces.nsIRDFResource);
+  if (!folderResource)
+     return false;
+
+  return (folderResource.Value == gCurrentFolderUri);
 }
 
 function OnLoadMessageWindow()
 {
 	HideMenus();
-  	AddMailOfflineObserver();
+  AddMailOfflineObserver();
 	CreateMailWindowGlobals();
 	CreateMessageWindowGlobals();
+  AddToolBarPrefListener();
+  ShowHideToolBarButtons()
 	verifyAccounts(null);
 
 	InitMsgWindow();
@@ -155,37 +261,85 @@ function OnLoadMessageWindow()
 
   try {
     var nsIFolderListener = Components.interfaces.nsIFolderListener;
-    var notifyFlags = nsIFolderListener.removed | nsIFolderListener.event;
+    var notifyFlags = nsIFolderListener.removed | nsIFolderListener.event | nsIFolderListener.intPropertyChanged;
     mailSession.AddFolderListener(folderListener, notifyFlags);
   } catch (ex) {
     dump("Error adding to session: " +ex + "\n");
   }
 
   var originalView = null;
+  var folder = null;
+  var messageUri;
+  var loadCustomMessage = false;       //set to true when either loading a message/rfc822 attachment or a .eml file
+  if (window.arguments)
+  {
+    if (window.arguments[0])
+    {
+      try
+      {
+        messageUri = window.arguments[0];
+        if (messageUri instanceof Components.interfaces.nsIURI)
+        {
+          loadCustomMessage = /type=x-message-display/.test(messageUri.spec);
+          gCurrentMessageUri = messageUri.spec;
+          if (messageUri instanceof Components.interfaces.nsIMsgMailNewsUrl)
+            folder = messageUri.folder;
+        }
+      } 
+      catch(ex) 
+      {
+        folder = null;
+        dump("## ex=" + ex + "\n");
+      }
 
-	if(window.arguments)
-	{
-		if(window.arguments[0])
-		{
-			gCurrentMessageUri = window.arguments[0];
-		}
-		else
-		{
-			gCurrentMessageUri = null;
-		}
+      if (!gCurrentMessageUri)
+        gCurrentMessageUri = window.arguments[0];
+    }
+    else
+      gCurrentMessageUri = null;
 
-		if(window.arguments[1])
-		{
-			gCurrentFolderUri = window.arguments[1];
-		}
-		else
-		{
-			gCurrentFolderUri = null;
-		}
+    if (window.arguments[1])
+      gCurrentFolderUri = window.arguments[1];
+    else
+      gCurrentFolderUri = folder ? folder.URI : null;
 
     if (window.arguments[2])
-      originalView = window.arguments[2];      
-	}	
+      originalView = window.arguments[2];
+
+  }
+
+  CreateView(originalView);
+
+  setTimeout(OnLoadMessageWindowDelayed, 0, loadCustomMessage);
+
+  SetupCommandUpdateHandlers();
+}
+
+function OnLoadMessageWindowDelayed(loadCustomMessage)
+{
+  if (loadCustomMessage)
+    gDBView.loadMessageByUrl(gCurrentMessageUri);
+  else
+  {
+    var msgKey = extractMsgKeyFromURI(gCurrentMessageUri);
+    gDBView.loadMessageByMsgKey(msgKey);
+  }
+  gNextMessageViewIndexAfterDelete = gDBView.msgToSelectAfterDelete; 
+  UpdateStandAloneMessageCounts();
+   
+  // set focus to the message pane
+  window.content.focus();
+
+  // since we just changed the pane with focus we need to update the toolbar to reflect this
+  // XXX TODO
+  // can we optimize
+  // and just update cmd_delete and button_delete?
+  UpdateMailToolbar("focus");
+}
+
+function CreateView(originalView)
+{
+  var msgFolder = GetLoadedMsgFolder();
 
   // extract the sort type, the sort order, 
   var sortType;
@@ -200,23 +354,37 @@ function OnLoadMessageWindow()
     sortType = originalView.sortType;
     sortOrder = originalView.sortOrder;
   }
-  
-  var msgFolder = GetLoadedMsgFolder();
-  CreateBareDBView(msgFolder,viewType, viewFlags, sortType, sortOrder); // create a db view for 
-
-  if (gCurrentMessageUri) {
-    SetUpToolbarButtons(gCurrentMessageUri);
+  else if (msgFolder)
+  {
+    var msgDatabase = msgFolder.getMsgDatabase(msgWindow);
+    if (msgDatabase)
+    {
+      var dbFolderInfo = msgDatabase.dBFolderInfo;
+      sortType = dbFolderInfo.sortType;
+      sortOrder = dbFolderInfo.sortOrder;
+      viewFlags = dbFolderInfo.viewFlags;
+      viewType = dbFolderInfo.viewType;
+      msgDatabase = null;
+      dbFolderInfo = null;
+   }
   }
-  else if (gCurrentFolderUri) {
-    SetUpToolbarButtons(gCurrentFolderUri);
-  }
- 
-  setTimeout("var msgKey = extractMsgKeyFromURI(gCurrentMessageUri); gDBView.loadMessageByMsgKey(msgKey); gNextMessageViewIndexAfterDelete = gDBView.msgToSelectAfterDelete;", 0);
 
-  SetupCommandUpdateHandlers();
-  var messagePaneFrame = top.frames['messagepane'];
-  if(messagePaneFrame)
-	messagePaneFrame.focus();
+  // create a db view
+  CreateBareDBView(originalView, msgFolder, viewType, viewFlags, sortType, sortOrder); 
+
+  var uri;
+  if (gCurrentMessageUri)
+    uri = gCurrentMessageUri;
+  else if (gCurrentFolderUri)
+    uri = gCurrentFolderUri;
+  else
+    uri = null;
+
+  SetUpToolbarButtons(uri);
+
+  // hook for extra toolbar items
+  var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
+  observerService.notifyObservers(window, "mail:setupToolbarItems", uri);
 }
 
 function extractMsgKeyFromURI()
@@ -236,53 +404,73 @@ function extractMsgKeyFromURI()
 function HideMenus()
 {
 	var message_menuitem=document.getElementById('menu_showMessage');
-	if(message_menuitem)
+	if (message_menuitem)
 		message_menuitem.setAttribute("hidden", "true");
 
+	var showSearchToolbar = document.getElementById('menu_showSearchToolbar');
+	if (showSearchToolbar)
+		showSearchToolbar.setAttribute("hidden", "true");
+
+	var showSearch_showMessage_Separator = document.getElementById('menu_showSearch_showMessage_Separator');
+	if (showSearch_showMessage_Separator)
+		showSearch_showMessage_Separator.setAttribute("hidden", "true");
+
 	var expandOrCollapseMenu = document.getElementById('menu_expandOrCollapse');
-	if(expandOrCollapseMenu)
+	if (expandOrCollapseMenu)
 		expandOrCollapseMenu.setAttribute("hidden", "true");
 
 	var renameFolderMenu = document.getElementById('menu_renameFolder');
-	if(renameFolderMenu)
+	if (renameFolderMenu)
 		renameFolderMenu.setAttribute("hidden", "true");
 
 	var viewMessagesMenu = document.getElementById('viewMessagesMenu');
-	if(viewMessagesMenu)
+	if (viewMessagesMenu)
 		viewMessagesMenu.setAttribute("hidden", "true");
 
+	var viewMessageViewMenu = document.getElementById('viewMessageViewMenu');
+	if (viewMessageViewMenu)
+		viewMessageViewMenu.setAttribute("hidden", "true");
+
+	var viewMessagesMenuSeparator = document.getElementById('viewMessagesMenuSeparator');
+	if (viewMessagesMenuSeparator)
+		viewMessagesMenuSeparator.setAttribute("hidden", "true");
+
 	var openMessageMenu = document.getElementById('openMessageWindowMenuitem');
-	if(openMessageMenu)
+	if (openMessageMenu)
 		openMessageMenu.setAttribute("hidden", "true");
 
 	var viewSortMenu = document.getElementById('viewSortMenu');
-	if(viewSortMenu)
+	if (viewSortMenu)
 		viewSortMenu.setAttribute("hidden", "true");
 
-	var viewThreadedMenu = document.getElementById('menu_showThreads');
-	if(viewThreadedMenu)
-		viewThreadedMenu.setAttribute("hidden", "true");
-
 	var emptryTrashMenu = document.getElementById('menu_emptyTrash');
-	if(emptryTrashMenu)
+	if (emptryTrashMenu)
 		emptryTrashMenu.setAttribute("hidden", "true");
 
 	var menuProperties = document.getElementById('menu_properties');
-	if(menuProperties)
+	if (menuProperties)
 		menuProperties.setAttribute("hidden", "true");
 
 	var compactFolderMenu = document.getElementById('menu_compactFolder');
-	if(compactFolderMenu)
+	if (compactFolderMenu)
 		compactFolderMenu.setAttribute("hidden", "true");
 
 	var trashSeparator = document.getElementById('trashMenuSeparator');
-	if(trashSeparator)
+	if (trashSeparator)
 		trashSeparator.setAttribute("hidden", "true");
 
+	var goStartPageSeparator = document.getElementById('goStartPageSeparator');
+	if (goStartPageSeparator)
+		goStartPageSeparator.hidden = true;
+
+  var goStartPage = document.getElementById('goStartPage');
+	if (goStartPage)
+   goStartPage.hidden = true;
 }
 
 function OnUnloadMessageWindow()
 {
+  RemoveToolBarPrefListener();
 	// FIX ME - later we will be able to use onunload from the overlay
 	OnUnloadMsgHeaderPane();
 
@@ -307,10 +495,9 @@ function GetSelectedMsgFolders()
 {
 	var folderArray = new Array(1);
 	var msgFolder = GetLoadedMsgFolder();
-	if(msgFolder)
-	{
+	if (msgFolder)
 		folderArray[0] = msgFolder;	
-	}
+
 	return folderArray;
 }
 
@@ -321,7 +508,7 @@ function GetFirstSelectedMessage()
 
 function GetNumSelectedMessages()
 {
-	if(gCurrentMessageUri)
+	if (gCurrentMessageUri)
 		return 1;
 	else
 		return 0;
@@ -331,9 +518,9 @@ function GetSelectedMessages()
 {
 	var messageArray = new Array(1);
 	var message = GetLoadedMessage();
-	if(message) {
+	if (message)
 		messageArray[0] = message;	
-	}
+
 	return messageArray;
 }
 
@@ -353,16 +540,10 @@ function GetSelectedIndices(dbView)
 
 function GetLoadedMsgFolder()
 {
-	if(gCurrentFolderUri)
-	{
-		var folderResource = RDF.GetResource(gCurrentFolderUri);
-		if(folderResource)
-		{
-			var msgFolder = folderResource.QueryInterface(Components.interfaces.nsIMsgFolder);
-			return msgFolder;
-		}
-	}
-	return null;
+	if (gCurrentFolderUri)
+		return RDF.GetResource(gCurrentFolderUri).QueryInterface(Components.interfaces.nsIMsgFolder);
+	else
+    return null;
 }
 
 function GetLoadedMessage()
@@ -390,16 +571,79 @@ function SetNextMessageAfterDelete()
 
 function SelectFolder(folderUri)
 {
-	gCurrentFolderUri = folderUri;
+  if (folderUri == gCurrentFolderUri)
+    return;
+
+  var msgfolder = RDF.GetResource(folderUri).QueryInterface(Components.interfaces.nsIMsgFolder);
+  if (!msgfolder || msgfolder.isServer)
+    return;
+
+  // close old folder view
+  var dbview = GetDBView();  
+  if (dbview)
+    dbview.close(); 
+
+  gCurrentFolderToRerootForStandAlone = folderUri;
+
+  if (msgfolder.manyHeadersToDownload)
+  {
+    gRerootOnFolderLoadForStandAlone = true;
+    try
+    {
+      msgfolder.startFolderLoading();
+      msgfolder.updateFolder(msgWindow);
+    }
+    catch(ex)
+    {
+      dump("Error loading with many headers to download: " + ex + "\n");
+    }
+  }
+  else
+  {
+    RerootFolderForStandAlone(folderUri);
+    gRerootOnFolderLoadForStandAlone = false;
+    msgfolder.startFolderLoading();
+
+    //Need to do this after rerooting folder.  Otherwise possibility of receiving folder loaded
+    //notification before folder has actually changed.
+    msgfolder.updateFolder(msgWindow);
+  }    
+}
+
+function RerootFolderForStandAlone(uri)
+{
+  gCurrentFolderUri = uri;
+
+  // create new folder view
+  CreateView(null);
+  
+  // now do the work to load the appropriate message
+  if (gNextMessageAfterLoad) {
+    var type = gNextMessageAfterLoad;
+    gNextMessageAfterLoad = null;
+    LoadMessageByNavigationType(type);
+  }
+  
+  SetUpToolbarButtons(gCurrentFolderUri);
+  
+  UpdateMailToolbar("reroot folder in stand alone window");
+  
+  // hook for extra toolbar items
+  var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
+  observerService.notifyObservers(window, "mail:setupToolbarItems", uri);
+} 
+
+function GetMsgHdrFromUri(messageUri)
+{
+  return messenger.messageServiceFromURI(messageUri).messageURIToMsgHdr(messageUri);
 }
 
 function SelectMessage(messageUri)
 {
-  var msgHdr = messenger.messageServiceFromURI(messageUri).messageURIToMsgHdr(messageUri)
-  gDBView.loadMessageByMsgKey(msgHdr.messageKey);
+  var msgHdr = GetMsgHdrFromUri(messageUri);
+  LoadMessageByMsgKey(msgHdr.messageKey);
 }
  
-
 function ReloadMessage()
 {
   gDBView.reloadMessage();
@@ -430,51 +674,39 @@ function MsgDeleteMessageFromMessageWindow(reallyDelete, fromToolbar)
 // MessageWindowController object (handles commands when one of the trees does not have focus)
 var MessageWindowController =
 {
-   supportsCommand: function(command)
-	{
-
-		switch ( command )
-		{
-			case "cmd_close":
-			case "cmd_reply":
-			case "button_reply":
-			case "cmd_replySender":
-			case "cmd_replyGroup":
-			case "cmd_replyall":
-			case "button_replyall":
-			case "cmd_forward":
-			case "button_forward":
-			case "cmd_forwardInline":
-			case "cmd_forwardAttachment":
-			case "cmd_editAsNew":
-      case "cmd_createFilterFromPopup":
-      case "cmd_createFilterFromMenu":
-			case "cmd_delete":
+  supportsCommand: function(command)
+  {
+    switch (command)
+    {
+      case "cmd_reply":
+      case "button_reply":
+      case "cmd_replySender":
+      case "cmd_replyGroup":
+      case "cmd_replyall":
+      case "button_replyall":
+      case "cmd_forward":
+      case "button_forward":
+      case "cmd_forwardInline":
+      case "cmd_forwardAttachment":
+      case "cmd_editAsNew":
+      case "cmd_delete":
       case "cmd_undo":
       case "cmd_redo":
       case "cmd_killThread":
       case "cmd_watchThread":
-			case "button_delete":
-			case "cmd_shiftDelete":
-      case "button_print":
-			case "cmd_print":
-		  case "cmd_printSetup":
-			case "cmd_saveAsFile":
-			case "cmd_saveAsTemplate":
-			case "cmd_viewPageSource":
-			case "cmd_reload":
-			case "cmd_getNewMessages":
-      case "button_getNewMessages":
+      case "button_delete":
+      case "button_junk":
+      case "cmd_shiftDelete":
+      case "cmd_saveAsFile":
+      case "cmd_saveAsTemplate":
+      case "cmd_viewPageSource":
       case "cmd_getMsgsForAuthAccounts":
-			case "cmd_getNextNMessages":
-			case "cmd_find":
-			case "cmd_findAgain":
-      case "cmd_search":
       case "button_mark":
-			case "cmd_markAsRead":
-			case "cmd_markAllRead":
-			case "cmd_markThreadAsRead":
-			case "cmd_markAsFlagged":
+      case "cmd_markAsRead":
+      case "cmd_markAllRead":
+      case "cmd_markThreadAsRead":
+      case "cmd_markReadByDate":
+      case "cmd_markAsFlagged":
       case "cmd_label0":
       case "cmd_label1":
       case "cmd_label2":
@@ -482,29 +714,50 @@ var MessageWindowController =
       case "cmd_label4":
       case "cmd_label5":
       case "button_file":
-			case "cmd_file":
-			case "cmd_settingsOffline":
-			case "cmd_nextMsg":
-      case "button_next":
-			case "cmd_nextUnreadMsg":
-			case "cmd_nextFlaggedMsg":
-			case "cmd_nextUnreadThread":
-			case "cmd_previousMsg":
-			case "cmd_previousUnreadMsg":
-			case "cmd_previousFlaggedMsg":
-				return true;
+      case "cmd_file":
+      case "cmd_markAsJunk":
+      case "cmd_markAsNotJunk":
+      case "cmd_applyFilters":
+      case "cmd_runJunkControls":
+      case "cmd_deleteJunk":
+      case "cmd_nextMsg":
+      case "button_next": 
+      case "cmd_nextUnreadMsg": 
+      case "cmd_nextFlaggedMsg": 
+      case "cmd_nextUnreadThread": 
+      case "cmd_previousMsg": 
+      case "cmd_previousUnreadMsg": 
+      case "cmd_previousFlaggedMsg":
+        return !(gDBView.keyForFirstSelectedMessage == nsMsgKey_None);
+      case "cmd_getNextNMessages":
+      case "cmd_find":
+      case "cmd_findAgain":
+      case "cmd_findPrev":
+      case "cmd_search":
+      case "cmd_reload":
+      case "cmd_getNewMessages":
+      case "button_getNewMessages":
+      case "button_print":
+      case "cmd_print":
+      case "cmd_printpreview":
+      case "cmd_printSetup":
+      case "cmd_close":
+      case "cmd_settingsOffline":
+      case "cmd_createFilterFromPopup":
+      case "cmd_createFilterFromMenu":
+        return true;
       case "cmd_synchronizeOffline":
-			case "cmd_downloadFlagged":
-			case "cmd_downloadSelected":
+      case "cmd_downloadFlagged":
+      case "cmd_downloadSelected":
         return CheckOnline();
-			default:
-				return false;
-		}
-	},
+      default:
+        return false;
+    }
+  },
 
 	isCommandEnabled: function(command)
 	{
-		switch ( command )
+		switch (command)
 		{
       case "cmd_createFilterFromPopup":
       case "cmd_createFilterFromMenu":
@@ -514,6 +767,17 @@ var MessageWindowController =
 			case "cmd_delete":
         UpdateDeleteCommand();
         // fall through
+			case "button_delete":
+			case "cmd_shiftDelete":
+        var loadedFolder = GetLoadedMsgFolder();
+        return gCurrentMessageUri && loadedFolder && (loadedFolder.canDeleteMessages || isNewsURI(gCurrentFolderUri));
+      case "button_junk":
+        UpdateJunkToolbarButton();
+        // fall through
+      case "cmd_markAsJunk":
+			case "cmd_markAsNotJunk":
+        // can't do junk on news yet
+        return (!isNewsURI(gCurrentFolderUri));
 			case "cmd_reply":
 			case "button_reply":
 			case "cmd_replySender":
@@ -525,9 +789,8 @@ var MessageWindowController =
 			case "cmd_forwardInline":
 			case "cmd_forwardAttachment":
 			case "cmd_editAsNew":
-			case "button_delete":
-			case "cmd_shiftDelete":
 			case "cmd_print":
+			case "cmd_printpreview":
       case "button_print":
 			case "cmd_saveAsFile":
 			case "cmd_saveAsTemplate":
@@ -538,6 +801,7 @@ var MessageWindowController =
 			case "cmd_markAsRead":
 			case "cmd_markAllRead":
 			case "cmd_markThreadAsRead":
+			case "cmd_markReadByDate":
       case "cmd_label0":
       case "cmd_label1":
       case "cmd_label2":
@@ -572,6 +836,7 @@ var MessageWindowController =
 			case "cmd_previousUnreadMsg":
 				return true;
 			case "cmd_findAgain":
+			case "cmd_findPrev":
 				return MsgCanFindAgain();
       case "cmd_search":
         var loadedFolder = GetLoadedMsgFolder();
@@ -580,7 +845,11 @@ var MessageWindowController =
         return loadedFolder.server.canSearchMessages; 
       case "cmd_undo":
       case "cmd_redo":
-                return SetupUndoRedoCommand(command);
+        return SetupUndoRedoCommand(command);
+      case "cmd_applyFilters":
+      case "cmd_runJunkControls":
+      case "cmd_deleteJunk":
+        return false;
 			default:
 				return false;
 		}
@@ -649,15 +918,21 @@ var MessageWindowController =
 			case "cmd_shiftDelete":
 				MsgDeleteMessageFromMessageWindow(true, false);
 				break;
+      case "button_junk":
+        MsgJunk();
+        break;
 			case "button_delete":
 				MsgDeleteMessageFromMessageWindow(false, true);
 				break;
 		  case "cmd_printSetup":
-		    goPageSetup();
+		    NSPrintSetup();
 		    break;
 			case "cmd_print":
 				PrintEnginePrint();
 				break;
+			case "cmd_printpreview":
+				PrintEnginePrintPreview();
+				break;
 			case "cmd_saveAsFile":
 				MsgSaveAsFile();
 				break;
@@ -674,7 +949,10 @@ var MessageWindowController =
 				MsgFind();
 				break;
 			case "cmd_findAgain":
-				MsgFindAgain();
+				MsgFindAgain(false);
+				break;
+			case "cmd_findPrev":
+				MsgFindAgain(true);
 				break;
       case "cmd_search":
         MsgSearchMessages();
@@ -689,9 +967,18 @@ var MessageWindowController =
 			case "cmd_markAllRead":
 				MsgMarkAllRead();
 				return;
+			case "cmd_markReadByDate":
+				MsgMarkReadByDate();
+				return;
 			case "cmd_markAsFlagged":
 				MsgMarkAsFlagged(null);
 				return;
+			case "cmd_markAsJunk":
+        JunkSelectedMessages(true);
+				return;
+			case "cmd_markAsNotJunk":
+        JunkSelectedMessages(false);
+				return;
       case "cmd_label0":
         gDBView.doCommand(nsMsgViewCommandType.label0);
  				return;
@@ -752,24 +1039,38 @@ var MessageWindowController =
 	}
 };
 
-function performNavigation(type)
+function LoadMessageByNavigationType(type)
 {
   var resultId = new Object;
   var resultIndex = new Object;
   var threadIndex = new Object;
 
   gDBView.viewNavigate(type, resultId, resultIndex, threadIndex, true /* wrap */);
-  
+
   // if we found something....display it.
   if ((resultId.value != nsMsgKey_None) && (resultIndex.value != nsMsgKey_None)) 
   {
     // load the message key
-    gDBView.loadMessageByMsgKey(resultId.value);
-    return;
+    LoadMessageByMsgKey(resultId.value);
+    // if we changed folders, the message counts changed.
+    UpdateStandAloneMessageCounts();
+
+    // new message has been loaded
+    return true;
   }
+
+  // no message found to load
+  return false;
+}
    
-  // we need to span another folder 
-  CrossFolderNavigation(type, false);
+function performNavigation(type)
+{
+  // Try to load a message by navigation type if we can find
+  // the message in the same folder.
+  if (LoadMessageByNavigationType(type))
+    return;
+   
+  CrossFolderNavigation(type);
 }
 
 function SetupCommandUpdateHandlers()
@@ -782,3 +1083,14 @@ function GetDBView()
   return gDBView;
 }
 
+function LoadMessageByMsgKey(messageKey)
+{
+  // we only want to update the toolbar if there was no previous selected message.
+  if (nsMsgKey_None == gDBView.keyForFirstSelectedMessage)
+  {
+    gDBView.loadMessageByMsgKey(messageKey);
+    UpdateMailToolbar("update toolbar for message Window");
+  }
+  else
+    gDBView.loadMessageByMsgKey(messageKey);
+}
\ No newline at end of file
